#include "scanlist.h"
#include <iwlib.h>
#include <unistd.h>
#include "aplistitem.h"

ScanList::ScanList()
{
	aplist.clear();
	dev_name = "";
	state = SCAN_NONE;

	pthread_mutex_init(&lock, NULL);
}

ScanList::~ScanList()
{
	pthread_mutex_destroy(&lock);
}

void ScanList::setDevice(const char *dev)
{
	dev_name = dev;
	pthread_mutex_lock(&lock);
	aplist.clear();
	pthread_mutex_unlock(&lock);
}

int ScanList::updateList()
{
	int skfd;
	int ret;
	wireless_scan_head scan_head;
	wireless_scan *scan;
	int we_version;
	char dev[128];

	if(dev_name.length() >= sizeof(dev))
		return RET_NG;

	skfd = iw_sockets_open();
	if(skfd < 0)
		return RET_NG;

	we_version = iw_get_kernel_we_version();

	strcpy(dev, dev_name.ascii());

	ret = iw_scan(skfd, dev, we_version, &scan_head);
	close(skfd);
	if(ret < 0)
		return RET_NG_SCAN;

	scan = scan_head.result;

	if(!scan)
		return RET_OK_NOSCAN;

	pthread_mutex_lock(&lock);
	aplist.clear();
	pthread_mutex_unlock(&lock);
	while(scan){
		wireless_scan *tmp;
		tmp = scan->next;

		if(scan->b.has_essid){
			scan->next = NULL;
			pthread_mutex_lock(&lock);
			aplist.append(*scan);
			pthread_mutex_unlock(&lock);
		}
		free(scan);
		scan = tmp;
	}

	return RET_OK_SCAN;
}

int ScanList::run()
{
	int ret;
	int count;
	cancel_flag = 0;

	if(dev_name.length() == 0)
		return 1;

	state = SCAN_INIT;
	pthread_mutex_lock(&lock);
	aplist.clear();
	pthread_mutex_unlock(&lock);

	ret = system("qcop QPE/Network 'start(QString,QString)' /opt/QtPalmtop/share/wificonnect/WLAN.conf ''");

	count = 10;
	do{
		usleep(1000000);
		if(cancel_flag)
			goto END;
		ret = updateList();
		if(ret == RET_NG)
			goto END;
	}while(count-- > 0 && ret != RET_OK_SCAN);

	state = SCAN_GET;
	count = 10;
	do{
		usleep(1000000);
		if(cancel_flag)
			goto END;
		ret = updateList();
		if(ret == RET_NG)
			goto END;
	}while(count-- > 0 && ret == RET_OK_SCAN);

  END:
	state = SCAN_POST;
	ret = system("qcop QPE/Network 'stop()'");
	state = SCAN_END;
	return 0;
}

ApList ScanList::getScanItems()
{
	pthread_mutex_lock(&lock);
	ApList tmp(aplist);
	pthread_mutex_unlock(&lock);

	return tmp;
}

int ScanList::getScanItemCount()
{
	int num;
	pthread_mutex_lock(&lock);
	num = aplist.count();
	pthread_mutex_unlock(&lock);

	return num;
}

int ScanList::getState()
{
	return (int)state;
}

void ScanList::cancel()
{
	if(tid){
		cancel_flag = 1;
	}
}
